
-- Default Config Values
ShotGlassSavedSettings = ShotGlassSavedSettings or {}
ShotGlassSavedAuras = ShotGlassSavedAuras or {}

ShotGlassDefaultSettings = {
	ShowDraggingHandle = true,		
	ShowSolo = true,							
	MirrorRow = false,				-- SetAttribute("point", "LEFT")
}

local function CreateCheckButton( reference, parent, label)
	local checkbutton = CreateFrame( "CheckButton", reference, parent, "InterfaceOptionsCheckButtonTemplate" )
	_G[reference.."Text"]:SetText(label)
	checkbutton.GetValue = function() if checkbutton:GetChecked() then return true else return false end end
	checkbutton.SetValue = checkbutton.SetChecked

	return checkbutton
end

local function CreateEditbox(reference, parent)
	local frame = CreateFrame("ScrollFrame", reference, parent, "UIPanelScrollFrameTemplate")
	frame.BorderFrame = CreateFrame("Frame", nil, frame )
	local EditBox = CreateFrame("EditBox", nil, frame)
	-- Margins	-- Bottom/Left are supposed to be negative
	frame.Margins = {Left = 4, Right = 24, Top = 8, Bottom = 8, }

	-- Frame Size
	frame:SetWidth(150)
	frame:SetHeight(100)
	-- Border
	frame.BorderFrame:SetPoint("TOPLEFT", 0, 5)
	frame.BorderFrame:SetPoint("BOTTOMRIGHT", 3, -5)
	frame.BorderFrame:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
										edgeFile = "Interface/Tooltips/UI-Tooltip-Border", 
										tile = true, tileSize = 16, edgeSize = 16, 
										insets = { left = 4, right = 4, top = 4, bottom = 4 }
										});
	frame.BorderFrame:SetBackdropColor(0.05, 0.05, 0.05, 0)
	frame.BorderFrame:SetBackdropBorderColor(0.5, 0.5, 0.5, 1)
	
	-- Text
	EditBox:SetPoint("TOPLEFT")
	EditBox:SetPoint("BOTTOMLEFT")
	EditBox:SetHeight(100)
	EditBox:SetWidth(150)
	EditBox:SetMultiLine(true)
			
	EditBox:SetFrameLevel(frame:GetFrameLevel()-1)
	EditBox:SetFont("Fonts\\FRIZQT__.TTF", 11, "NONE")
	EditBox:SetText("Empty")
	EditBox:SetAutoFocus(false)
	EditBox:SetTextInsets(9, 6, 2, 2)
	frame:SetScrollChild(EditBox)
	frame.EditBox = EditBox
	
	function frame:GetValue() return EditBox:GetText() end
	function frame:SetValue(value) EditBox:SetText(value) end
	frame._SetWidth = frame.SetWidth
	function frame:SetWidth(value) frame:_SetWidth(value); EditBox:SetWidth(value) end
	
	return frame
end
	

local function ToggleDraggingHandle(show)
	if show == false then ShotGlassRaidFrames.HandleFrame:Hide() 
	else ShotGlassRaidFrames.HandleFrame:Show() end
end

local function ApplyConfiguration()
	
	ToggleDraggingHandle(ShotGlassSavedSettings.ShowDraggingHandle)
	if not InCombatLockdown() then
		ShotGlassRaidFrames:SetAttribute("showSolo", ShotGlassSavedSettings.ShowSolo)
	end
end

local function ResetPosition()
	print("Resetting Shot Glass position...")
	ShotGlassRaidFrames:ClearAllPoints()
	ShotGlassRaidFrames:SetPoint("CENTER", UIParent)
end

local function AddAuras()
	for i,v in pairs(ShotGlassSavedAuras) do
		TrackedAuras[i] = {position = v.position, color = v.color,}
	end
end


local function CreateInterfacePanel( objectName, panelTitle)
	
	-- Panel
	------------------------------
	local panel = CreateFrame( "Frame", objectName.."_InterfaceOptionsPanel", UIParent);

	-- controls
	local controls = {}
	
	local function GetPanelValues()
		for i,v in pairs(controls) do
			if v.GetValue then ShotGlassSavedSettings[i] = v:GetValue() end
		end
		ApplyConfiguration()
	end
	
	local function SetPanelValues()
		for i,v in pairs(controls) do
			if v.SetValue then v:SetValue(ShotGlassSavedSettings[i]) end
		end
	end
	
	panel.name = panelTitle
	panel.okay = GetPanelValues
	panel.refresh = SetPanelValues
	
	panel:SetScript("OnEvent", function(self, event)
		if event == "PLAYER_ENTERING_WORLD" then
			AddAuras()
		
			ShotGlassSavedSettings = ShotGlassSavedSettings or {}
			for i,v in pairs(ShotGlassDefaultSettings) do
				ShotGlassSavedSettings[i] = ShotGlassSavedSettings[i] or ShotGlassDefaultSettings[i]
			end
		end
		
	end)
	panel:RegisterEvent("PLAYER_ENTERING_WORLD")

	local version = GetAddOnMetadata("ShotGlass", "version")
	
	panel.MainLabel = panel:CreateFontString(nil, 'ARTWORK', 'GameFontNormalLarge')
	panel.MainLabel:SetPoint("TOP", panel, "TOP", 0, -16)
	panel.MainLabel:SetPoint("LEFT", panel, "LEFT", 16, 0)
	panel.MainLabel:SetHeight(15)
	panel.MainLabel:SetWidth(350)
	panel.MainLabel:SetJustifyH("LEFT")
	panel.MainLabel:SetJustifyV("TOP")
	panel.MainLabel:SetText(panelTitle.." "..version)
	
	controls.ShowDraggingHandle = CreateCheckButton(objectName.."ShowDraggingHandle", panel, "Show the Drag Handle")
	controls.ShowDraggingHandle:SetPoint("TOP", panel.MainLabel, "TOP", 0, -32)
	controls.ShowDraggingHandle:SetPoint("LEFT", panel, "LEFT", 24, 0)
	
	controls.ShowSolo = CreateCheckButton(objectName.."ShowSolo", panel, "Show Raid Frame when Solo")
	controls.ShowSolo:SetPoint("TOP", controls.ShowDraggingHandle, "TOP", 0, -28)
	controls.ShowSolo:SetPoint("LEFT", panel, "LEFT", 24, 0)
	
	-- Reset
	ResetButton = CreateFrame("Button", objectName.."ResetButton", panel, "UIPanelButtonTemplate")
	ResetButton:SetPoint("TOP", controls.ShowSolo, "TOP", 0, -28)
	ResetButton:SetPoint("LEFT", panel, "LEFT", 24, 0)
	ResetButton:SetWidth(300)
	ResetButton:SetText("Reset Position")
	ResetButton:SetScript("OnClick", ResetPosition)
	
	--[[
	local CodeEditor = CreateEditbox(objectName.."CodeEditor", panel)
	CodeEditor:SetPoint("TOP", ResetButton, "BOTTOM", 0, -28)
	CodeEditor:SetPoint("LEFT", panel, "LEFT", 24, 0)
	CodeEditor:SetWidth(250)
	CodeEditor:SetHeight(300)
	--]]
	
	
	
	-- Set Click for ALL Controls
	for i,v in pairs(controls) do
		v:SetScript("OnClick", GetPanelValues)
	end
	
	----------------
	-- Return a pointer to the whole thingy
	----------------
	InterfaceOptions_AddCategory(panel)
	return panel
end

CreateInterfacePanel("ShotGlassPanel", "Shot Glass")

----------------------------------------
-- Command Line Interpreter
----------------------------------------
local CommandLineList = {
	["SHOW"] = function() 
		ShotGlassRaidFrames:Show()
	end,
	["HIDE"] = function() 
		ShotGlassRaidFrames:Hide()
	end,
	["LOCK"] = function() 
		ShotGlassSavedSettings.ShowDraggingHandle = false
		ToggleDraggingHandle(ShotGlassSavedSettings.ShowDraggingHandle)
	end,
	["UNLOCK"] = function() 
		ShotGlassSavedSettings.ShowDraggingHandle = true
		ToggleDraggingHandle(ShotGlassSavedSettings.ShowDraggingHandle)
	end,
}

local function ParseCommandLine(text)
	if text then 
		local command, param1, param2, param3 = text:upper():match("^([%S]+)%s*([%S]*)%s*(.*)$");
		if command and CommandLineList[command] then
			CommandLineList[command](param1, param2, param3)
		else InterfaceOptionsFrame_OpenToCategory("Shot Glass") end
	end
end

SLASH_SHOTGLASS1 = "/shotglass"
SlashCmdList["SHOTGLASS"] = ParseCommandLine

----------------------------------------
-- Optional Functions (for Tracked Auras)
----------------------------------------
do
	local Colors = {}
	Colors.Grey = {r = .5, g = .5, b = .5,}
	Colors.Red = {r = 1, g = 0, b = 0,}
	Colors.Orange = {r = 1, g = .5, b = 0,}
	Colors.Yellow = {r = 1, g = 1, b = 0,}
	Colors.Green = {r = 0, g = 1, b = 0,}

	local function TextFunction(aura)
		local text, textcolor
		local timeRemaining = aura.expiration - GetTime()

		if timeRemaining < 6  then 
			-- Time Remaining
			text = ceil(timeRemaining)
			if timeRemaining > 4 then textcolor = Colors.Green
			elseif timeRemaining > 2 then textcolor = Colors.Yellow
			else textcolor = Colors.Red end
		else
			-- Stacks
			if aura.stacks and (aura.stacks > 0) then
				text = aura.stacks
				textcolor = Colors.Grey
			else
				text = ""
				textcolor = Colors.Grey
			end
		end	
		return text, textcolor
	end

	ShotGlassRaidFrames.TextCoutdownFunction = TextFunction
end
